@@ -1,22 +1,11 @@
-/*************************************************************************
-  *  Copyright (C) 2017 by Andrius Štikonas <andrius@stikonas.eu>         *
- *                                                                       *
- *  This program is free software; you can redistribute it and/or        *
- *  modify it under the terms of the GNU General Public License as       *
- *  published by the Free Software Foundation; either version 3 of       *
- *  the License, or (at your option) any later version.                  *
- *                                                                       *
- *  This program is distributed in the hope that it will be useful,      *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
- *  GNU General Public License for more details.                         *
- *                                                                       *
- *  You should have received a copy of the GNU General Public License    *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
- *************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2017-2020 Andrius Štikonas <andrius@stikonas.eu>
+    SPDX-FileCopyrightText: 2018 Caio Jordão Carvalho <caiojcarvalho@gmail.com>
 
-#if !defined(SFDISKBACKEND__H)
+    SPDX-License-Identifier: GPL-3.0-or-later
+*/
 
+#ifndef SFDISKBACKEND__H
 #define SFDISKBACKEND__H
 
 #include "backend/corebackend.h"
@@ -27,6 +16,8 @@
 #include <QVariant>
 
 class Device;
+class ExternalCommand;
+class Partition;
 class KPluginFactory;
 class QString;
 
@@ -59,8 +50,13 @@ public:
 private:
     static void readSectorsUsed(const Device& d, Partition& p, const QString& mountPoint);
     void scanDevicePartitions(Device& d, const QJsonArray& jsonPartitions);
+    Partition* scanPartition(Device& d, const QString& partitionNode, const qint64 firstSector, const qint64 lastSector, const QString& partitionType, const bool bootable);
+    void scanWholeDevicePartition(Device& d);
+    static void setupPartitionInfo(const Device& d, Partition* partition, const QJsonObject& partitionObject);
     bool updateDevicePartitionTable(Device& d, const QJsonObject& jsonPartitionTable);
     static PartitionTable::Flags availableFlags(PartitionTable::TableType type);
+    static FileSystem::Type fileSystemNameToType(const QString& fileSystemName, const QString& version);
+    static FileSystem::Type runDetectFileSystemCommand(ExternalCommand& command, QString& typeRegExp, QString& versionRegExp, QString& name);
 };
 
 #endif
